/*###############################################################################
## OpenBTrainer project
##
## Copyright (C) 2009 Jose Luis Falcón Ramírez  jose.falconra@alum.uca.es
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################*/


#include <SDL/SDL.h>
#include <SDL/SDL_mixer.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_ttf.h>
#include <iostream>
#include "juego.h"
#include "minijuego.h"
#include "rapidez.h"
#include "memoria.h"
#include "calculo.h"
#include "logica.h"


using namespace std;

juego::juego() {
    if ( SDL_Init( SDL_INIT_VIDEO|SDL_INIT_AUDIO|SDL_INIT_TIMER) < 0 )
    {
        cerr<<"Unable to init SDL: %s\n"<< SDL_GetError() <<endl;
        exit(1);
    }
    if(Mix_OpenAudio(22050, MIX_DEFAULT_FORMAT,1, 2048) < 0) {
        cerr << "Subsistema de Audio no disponible" << endl;
        exit(1);
    }
    atexit(Mix_CloseAudio);
    srand(time(0));
    //Cargamos las fuentes
    if(TTF_Init() == 0) {
        atexit(TTF_Quit);
    }
    SDL_EnableUNICODE(1);

}

juego::juego(const juego& orig) {
}

SDL_Surface* juego::iniciar_pantalla(){
    SDL_Surface* pantalla;
    pantalla=SDL_SetVideoMode(800,600,16,SDL_HWSURFACE|SDL_DOUBLEBUF);
    if(!pantalla)
    {
        cerr<<"Imposible crear 800x600 video: "<<SDL_GetError()<<endl;
        exit(1);
    }
    return pantalla;
}

void juego::minijuegos(perfiles *perfil, Musica m, SDL_Surface* pantalla){
    minijuego minigame;
    carga: SDL_Surface* bmp = IMG_Load("../imagenes/fondos/minijuegos.png");
    if (!bmp)
    {
        cerr<<"Imposible cargar bitmap: "<< SDL_GetError()<<endl;
        exit(1);
    }
    SDL_BlitSurface(bmp,0,pantalla,0);
     SDL_Surface* boton1 = IMG_Load("../imagenes/botones/Memorion.png");
    if(!boton1)
    {
        cerr<<"Imposible cargar bitmap: "<< SDL_GetError()<<endl;
        exit(1);
    }
    SDL_Rect rectbtn;
    rectbtn.x=500;
    rectbtn.y=180;
    SDL_BlitSurface(boton1, 0, pantalla, &rectbtn);
     SDL_Surface* boton2 = IMG_Load("../imagenes/botones/Diferencias.png");
    if(!boton2)
    {
        cerr<<"Imposible cargar bitmap: "<< SDL_GetError()<<endl;
        exit(1);
    }
    rectbtn.x=500;
    rectbtn.y=240;
    SDL_BlitSurface(boton2, 0, pantalla, &rectbtn);
    SDL_Surface* boton3 = IMG_Load("../imagenes/botones/Volver.png");
    if(!boton3)
    {
        cerr<<"Imposible cargar bitmap: "<< SDL_GetError()<<endl;
        exit(1);
    }
    rectbtn.x=630;
    rectbtn.y=20;
    SDL_BlitSurface(boton3, 0, pantalla, &rectbtn);
    bool done = false;
    //Bucle de minijuegos
    while (!done)
    {
        // Manejo de eventos
        SDL_Event event;
        while(SDL_PollEvent(&event)){
        	//tecl.actualizar();
       		if(event.type==SDL_QUIT){
        		done=true;
        		break;
       		 }
                if(event.type==SDL_MOUSEBUTTONDOWN)
                {
                    Uint32 mousex, mousey;
                    mousex=event.button.x;
                    mousey=event.button.y;
                    if(mousex>500&&mousey>180&&mousex<(500+150)&&mousey<(180+40)){
                        cout<<"Pulsacion de boton: Memorion"<<endl;
                        m.Pausar();
                        perfil->actualizarpuntuacion(perfil->perfilactual(),1,minigame.memorion(pantalla),2,pantalla);
                        m.Reproducir();
                        goto carga;
                    }
                    if(mousex>500&&mousey>240&&mousex<(500+150)&&mousey<(240+40)){
                        cout<<"Pulsacion de boton: Diferencias"<<endl;
                        m.Pausar();
                        perfil->actualizarpuntuacion(perfil->perfilactual(),2,minigame.diferencias(pantalla),2,pantalla);
                        m.Reproducir();
                        goto carga;
                    }
                    if(mousex>630&&mousey>20&&mousex<(630+150)&&mousey<(20+40)){
                        cout<<"Pulsacion de boton: Volver"<<endl;
                        done=true;
                    }
                }
        }
        SDL_Flip(pantalla);
    }
}

void juego::DibujarEscenarioCategorias(SDL_Surface* pantalla){
    SDL_Surface* bmp = IMG_Load("../imagenes/fondos/categorias.png");
    if (!bmp)
    {
        cerr<<"Imposible cargar bitmap: "<< SDL_GetError()<<endl;
        exit(1);
    }
    SDL_BlitSurface(bmp,0,pantalla,0);
    SDL_Surface* boton1 = IMG_Load("../imagenes/botones/Logica.png");
    if(!boton1)
    {
        cerr<<"Imposible cargar bitmap: "<< SDL_GetError()<<endl;
        exit(1);
    }
    SDL_Rect rectbtn;
    rectbtn.x=85;
    rectbtn.y=180;
    SDL_BlitSurface(boton1, 0, pantalla, &rectbtn);
    SDL_Surface* boton2 = IMG_Load("../imagenes/botones/Memoria.png");
    if(!boton2)
    {
        cerr<<"Imposible cargar bitmap: "<< SDL_GetError()<<endl;
        exit(1);
    }
    rectbtn.x=245;
    rectbtn.y=180;
    SDL_BlitSurface(boton2, 0, pantalla, &rectbtn);
    SDL_Surface* boton3 = IMG_Load("../imagenes/botones/RapidezVisual.png");
    if(!boton3)
    {
        cerr<<"Imposible cargar bitmap: "<< SDL_GetError()<<endl;
        exit(1);
    }
    rectbtn.x=405;
    rectbtn.y=180;
    SDL_BlitSurface(boton3, 0, pantalla, &rectbtn);
    SDL_Surface* boton4 = IMG_Load("../imagenes/botones/Calculo.png");
    if(!boton4)
    {
        cerr<<"Imposible cargar bitmap: "<< SDL_GetError()<<endl;
        exit(1);
    }
    rectbtn.x=565;
    rectbtn.y=180;
    SDL_BlitSurface(boton4, 0, pantalla, &rectbtn);
    SDL_Surface* boton5 = IMG_Load("../imagenes/botones/Volver.png");
    if(!boton5)
    {
        cerr<<"Imposible cargar bitmap: "<< SDL_GetError()<<endl;
        exit(1);
    }
    rectbtn.x=630;
    rectbtn.y=20;
    SDL_BlitSurface(boton5, 0, pantalla, &rectbtn);
}

void juego::DesplegarBotones(Uint32 categoria, SDL_Surface* pantalla){
    SDL_Rect rectbtn;
    switch(categoria){
        case 1:{
            //Carga de boton efecto pulsado
            SDL_Surface* boton1 = IMG_Load("../imagenes/botones/Logica_p.png");
            if(!boton1)
            {
                cerr<<"Imposible cargar bitmap: "<< SDL_GetError()<<endl;
                exit(1);
            }
            
            rectbtn.x=85;
            rectbtn.y=180;
            SDL_BlitSurface(boton1, 0, pantalla, &rectbtn);
            //Despliego de botones
             SDL_Surface* boton3 = IMG_Load("../imagenes/botones/Luces_y_Sombras.png");
            if(!boton3)
            {
                cerr<<"Imposible cargar bitmap: "<< SDL_GetError()<<endl;
                exit(1);
            }
            rectbtn.x=85;
            rectbtn.y=240;
            SDL_BlitSurface(boton3, 0, pantalla, &rectbtn);
        }
        break;
        case 2:{
            //Carga de boton efecto pulsado
            SDL_Surface* boton2 = IMG_Load("../imagenes/botones/Memoria_p.png");
            if(!boton2)
            {
                cerr<<"Imposible cargar bitmap: "<< SDL_GetError()<<endl;
                exit(1);
            }
            rectbtn.x=245;
            rectbtn.y=180;
            SDL_BlitSurface(boton2, 0, pantalla, &rectbtn);
            //Despliego de botones
             SDL_Surface* boton3 = IMG_Load("../imagenes/botones/Acuerdatedemi­.png");
            if(!boton3)
            {
                cerr<<"Imposible cargar bitmap: "<< SDL_GetError()<<endl;
                exit(1);
            }
            rectbtn.x=245;
            rectbtn.y=240;
            SDL_BlitSurface(boton3, 0, pantalla, &rectbtn);
        }
        break;
        case 3:{
            //Carga de boton efecto pulsado
            SDL_Surface* boton3 = IMG_Load("../imagenes/botones/RapidezVisual_p.png");
            if(!boton3)
            {
                cerr<<"Imposible cargar bitmap: "<< SDL_GetError()<<endl;
                exit(1);
            }
            rectbtn.x=405;
            rectbtn.y=180;
            SDL_BlitSurface(boton3, 0, pantalla, &rectbtn);
            //Despliego de botones
            SDL_Surface* boton4 = IMG_Load("../imagenes/botones/Observacion.png");
            if(!boton4)
            {
                cerr<<"Imposible cargar bitmap: "<< SDL_GetError()<<endl;
                exit(1);
            }
            rectbtn.x=405;
            rectbtn.y=240;
            SDL_BlitSurface(boton4, 0, pantalla, &rectbtn);
            SDL_Surface* boton5 = IMG_Load("../imagenes/botones/Recuerda.png");
            if(!boton5)
            {
                cerr<<"Imposible cargar bitmap: "<< SDL_GetError()<<endl;
                exit(1);
            }
            rectbtn.x=405;
            rectbtn.y=300;
            SDL_BlitSurface(boton5, 0, pantalla, &rectbtn);
            SDL_Surface* boton6 = IMG_Load("../imagenes/botones/Ojodecerradura.png");
            if(!boton6)
            {
                cerr<<"Imposible cargar bitmap: "<< SDL_GetError()<<endl;
                exit(1);
            }
            rectbtn.x=405;
            rectbtn.y=360;
            SDL_BlitSurface(boton6, 0, pantalla, &rectbtn);
        }
        break;
        case 4:{
            //Carga de boton efecto pulsado
            SDL_Surface* boton3 = IMG_Load("../imagenes/botones/Calculo_p.png");
            if(!boton3)
            {
                cerr<<"Imposible cargar bitmap: "<< SDL_GetError()<<endl;
                exit(1);
            }
            rectbtn.x=565;
            rectbtn.y=180;
            SDL_BlitSurface(boton3, 0, pantalla, &rectbtn);
            //Despliego de botones
            SDL_Surface* boton4 = IMG_Load("../imagenes/botones/Aritmetica.png");
            if(!boton4)
            {
                cerr<<"Imposible cargar bitmap: "<< SDL_GetError()<<endl;
                exit(1);
            }
            rectbtn.x=565;
            rectbtn.y=240;
            SDL_BlitSurface(boton4, 0, pantalla, &rectbtn);
            SDL_Surface* boton5 = IMG_Load("../imagenes/botones/Operadores.png");
            if(!boton5)
            {
                cerr<<"Imposible cargar bitmap: "<< SDL_GetError()<<endl;
                exit(1);
            }
            rectbtn.x=565;
            rectbtn.y=300;
            SDL_BlitSurface(boton5, 0, pantalla, &rectbtn);
        }
        break;
        default:
            cout<<"Error categoría inválida"<<endl;
            break;
    }
}

void juego::Categorias(perfiles *perfil, Musica m,SDL_Surface* pantalla)
{
    rapidez categorias;
    memoria memo;
    calculo calc;
    logica log;
    bool done=false;
    bool desplegadoCategoria[4];
    cout<<"Perfil activo: "<<perfil->perfilactual()<<endl;
    //Iniciamos con todas las categorias plegadas
    for(int i=0;i<4;i++)
        desplegadoCategoria[i]=false;
    //Dibujamos el escenario con sus botones iniciales
    DibujarEscenarioCategorias(pantalla);
    //Bucle de Categorías
    while (!done)
    {
        // Manejo de eventos
        SDL_Event event;
        while(SDL_PollEvent(&event)){
        	//tecl.actualizar();
       		if(event.type==SDL_QUIT){
        		done=true;
        		break;
       		 }
                if(event.type==SDL_MOUSEBUTTONDOWN)
                {
                    Uint32 mousex, mousey;
                    mousex=event.button.x;
                    mousey=event.button.y;
                    //Comprobacion btn modo arcade
                    if(mousex>85&&mousey>180&&mousex<(85+150)&&mousey<(180+40)){
                        cout<<"Pulsacion de boton: Logica"<<endl;
                        if(desplegadoCategoria[0]==true){
                            DibujarEscenarioCategorias(pantalla);
                            desplegadoCategoria[0]=false;
                        }
                        else{
                            DesplegarBotones(1,pantalla);
                            desplegadoCategoria[0]=true;
                        }
                    }
                    if(mousex>245&&mousey>180&&mousex<(245+150)&&mousey<(180+40)){
                        cout<<"Pulsacion de boton: Memoria"<<endl;
                        if(desplegadoCategoria[1]==true){
                            DibujarEscenarioCategorias(pantalla);
                            desplegadoCategoria[1]=false;
                        }
                        else{
                            DesplegarBotones(2,pantalla);
                            desplegadoCategoria[1]=true;
                        }
                    }
                    if(mousex>405&&mousey>180&&mousex<(405+150)&&mousey<(180+40)){
                        cout<<"Pulsacion de boton: Rapidez Visual"<<endl;
                        if(desplegadoCategoria[2]==true){
                            DibujarEscenarioCategorias(pantalla);
                            desplegadoCategoria[2]=false;
                        }
                        else{
                            DesplegarBotones(3,pantalla);
                            desplegadoCategoria[2]=true;
                        }
                    }
                    if(mousex>565&&mousey>180&&mousex<(565+150)&&mousey<(180+40)){
                        cout<<"Pulsacion de boton: Calculo"<<endl;
                        if(desplegadoCategoria[3]==true){
                            DibujarEscenarioCategorias(pantalla);
                            desplegadoCategoria[3]=false;
                        }
                        else{
                            DesplegarBotones(4,pantalla);
                            desplegadoCategoria[3]=true;
                        }
                    }
                    if(mousex>630&&mousey>20&&mousex<(630+150)&&mousey<(20+40)){
                        cout<<"Pulsacion de boton: Volver"<<endl;
                        done=true;
                    }
                    //Si hay algun menu desplegado...
                    if(desplegadoCategoria[0]==true){
                        if(mousex>85&&mousey>240&&mousex<(85+150)&&mousey<(240+40)){
                            cout<<"Pulsacion de boton: Luces y Sombras"<<endl;
                            m.Pausar();
                            perfil->actualizarpuntuacion(perfil->perfilactual(),8,log.lucesysombras(1,false,pantalla),2,pantalla);
                            m.Reproducir();
                            desplegadoCategoria[0]=false;
                            DibujarEscenarioCategorias(pantalla);
                        }
                    }
                    if(desplegadoCategoria[1]==true){
                        if(mousex>245&&mousey>240&&mousex<(245+150)&&mousey<(240+40)){
                            cout<<"Pulsacion de boton: Acuerdatedemi"<<endl;
                            m.Pausar();
                            perfil->actualizarpuntuacion(perfil->perfilactual(),6,memo.acuerdatedemi(pantalla),1,pantalla);
                            m.Reproducir();
                            desplegadoCategoria[1]=false;
                            DibujarEscenarioCategorias(pantalla);
                        }
                    }
                    if(desplegadoCategoria[2]==true){
                        if(mousex>405&&mousey>240&&mousex<(405+150)&&mousey<(240+40)){
                            cout<<"Pulsacion de boton: Observacion"<<endl;
                            m.Pausar();
                            perfil->actualizarpuntuacion(perfil->perfilactual(),4,categorias.observa(pantalla),1,pantalla);
                            m.Reproducir();
                            desplegadoCategoria[2]=false;
                            DibujarEscenarioCategorias(pantalla);
                        }
                        if(mousex>405&&mousey>300&&mousex<(405+150)&&mousey<(300+40)){
                            cout<<"Pulsacion de boton: Recuerda Digitos"<<endl;
                            m.Pausar();
                            perfil->actualizarpuntuacion(perfil->perfilactual(),3,categorias.recuerda(pantalla),1,pantalla);
                            m.Reproducir();
                            desplegadoCategoria[2]=false;
                            DibujarEscenarioCategorias(pantalla);
                        }
                        if(mousex>405&&mousey>360&&mousex<(405+150)&&mousey<(360+40)){
                            cout<<"Pulsacion de boton: Ojo de Cerradura"<<endl;
                            m.Pausar();
                            perfil->actualizarpuntuacion(perfil->perfilactual(),5,categorias.ojodecerradura(pantalla),1,pantalla);
                            m.Reproducir();
                            desplegadoCategoria[2]=false;
                            DibujarEscenarioCategorias(pantalla);
                        }
                    }
                    if(desplegadoCategoria[3]==true){
                        if(mousex>565&&mousey>240&&mousex<(565+150)&&mousey<(240+40)){
                            cout<<"Pulsacion de boton: Aritmetica"<<endl;
                            m.Pausar();
                            perfil->actualizarpuntuacion(perfil->perfilactual(),7,calc.aritmetica(pantalla),1,pantalla);
                            m.Reproducir();
                            desplegadoCategoria[3]=false;
                            DibujarEscenarioCategorias(pantalla);
                        }
                        if(mousex>565&&mousey>300&&mousex<(565+150)&&mousey<(300+40)){
                            cout<<"Pulsacion de boton: Operadores"<<endl;
                            m.Pausar();
                            perfil->actualizarpuntuacion(perfil->perfilactual(),10,calc.operadores(pantalla),1,pantalla);
                            m.Reproducir();
                            desplegadoCategoria[3]=false;
                            DibujarEscenarioCategorias(pantalla);
                        }
                    }
                }
        }
        SDL_Flip(pantalla);
    }
}

void juego::Arcade(perfiles* perfil, Musica m, SDL_Surface* pantalla){
    rapidez categorias;
    memoria memo;
    calculo calc;
    logica log;
    Uint32 puntos, puntostotales;
    m.Pausar();
    puntos=categorias.recuerda(pantalla);
    puntostotales=puntos*100;
    puntos=memo.acuerdatedemi(pantalla);
    puntostotales+=puntos*100;
    puntos=calc.aritmetica(pantalla);
    puntostotales+=puntos*10;
    puntos=log.lucesysombras(2,true,pantalla);
    puntostotales-=puntos*3;
    perfil->actualizarpuntuacion(perfil->perfilactual(),9,puntostotales,3,pantalla);
    m.Reproducir();
}
