/*###############################################################################
## OpenBTrainer project
##
## Copyright (C) 2009 Jose Luis Falcón Ramírez  jose.falconra@alum.uca.es
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################*/

#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#ifndef _RELOJ_H
#define	_RELOJ_H
/** Clase reloj
 * Esta clase nos resultará muy útil cada vez que deseemos controlar el tiempo
 * desde cualquier parte del juego
 */
class reloj {
public:
    /** Constructor de la clase reloj.
     * Si optamos por usar este tipo de constructor estaremos creando un reloj e irá incrementando el tiempo
     * @param posx le pasaremos la posicion del eje x donde quiera que se muestre el reloj
     * @param posy le pasaremos la posicion del eje y donde quiera que se muestre el reloj
     */
    reloj(Uint32 posx, Uint32 posy);
    /** Constructor de la clase reloj.
     * Si optamos por usar este tipo de reloj crearemos una cuenta atrás con el tiempo que le pasamos como parámetro
     * @param tiempo le pasamos como parametro el tiempo del que deseamos que se cree la cuenta atrás
     * @param posx le pasaremos la posicion del eje x donde quiera que se muestre el reloj
     * @param posy le pasaremos la posicion del eje y donde quiera que se muestre el reloj
     */
    reloj(Uint32 tiempo, Uint32 posx, Uint32 posy);
    /** Con esta funcion actualizamos el tiempo para que se incremente el tiempo en el primer tipo de reloj o para que
     * se decremente si estamos en un reloj de tipo cuenta atras
     * @param pantalla pasamos como parametro la variable pantalla ya que desde dentro de la función se llamará a la funcion dibuja tiempo para que se imprima el tiempo en la posicion que dimos en el constructor
     */
    void actualizartiempo(SDL_Surface* pantalla);
    /** Con esta funcion consultamos el tiempo en cualquier momento
     * @param min le pasamos una referencia a un entero donde se almacenará el valor de minutos del reloj
     * @param seg le pasamos una referencua a un entero donde se almacenará el valor de los segundos del reloj
     */
    void consultartiempo(Uint32 *min, Uint32 *seg);
    /** Con esta funcion consultamos los segundos. Nos será realmente útil si el reloj es de tipo cuenta atrás para controlar desde el juego que se ha llegado a 0 segundos
    */
    Uint32 consultarseg();
    /** Con esta funcion imprimimos el valor del tiempo en la posicion que tiene como atributo la clase
     * @param pantalla recibe la variable pantalla para que la funcion imprima directamente en la pantalla
     */
    
    void dibujatiempo(SDL_Surface* pantalla);
    /**Esta función sirve para asignar a un reloj tipo cuenta atrás unos determinados minutos y segundos
     * @param min le pasamos la cantidad de minutos que poseerá el reloj.
     * @param seg le pasamos la cantidad de segundos que poseerá el reloj.*/
    void settime(Uint32 min, Uint32 seg);
    virtual ~reloj();
private:
    Uint32  minutos, segundos, _posx, _posy;
    TTF_Font *fuente;
    bool cronometro;
    SDL_Surface *texto;
};

#endif	/* _RELOJ_H */

